/*
 * DBeaver - Universal Database Manager
 * Copyright (C) 2010-2016 Serge Rieder (serge@jkiss.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 2)
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jkiss.dbeaver.model.exec;

import org.jkiss.code.NotNull;
import org.jkiss.dbeaver.model.DBPCloseableObject;
import org.jkiss.dbeaver.model.DBPDataSource;
import org.jkiss.dbeaver.model.data.DBDPreferences;
import org.jkiss.dbeaver.model.runtime.DBRProgressMonitor;

/**
 * Execution session
 */
public interface DBCSession extends DBPCloseableObject, DBDPreferences {

    /**
     * Session title
     * @return title
     */
    @NotNull
    String getTaskTitle();

    /**
     * Data source of this session
     * @return data source
     */
    @NotNull
    DBCExecutionContext getExecutionContext();

    /**
     * Data source of this session
     * @return data source
     */
    @NotNull
    DBPDataSource getDataSource();

    /**
     * Performs check that this context is really connected to remote database
     * @return connected state
     */
    boolean isConnected();

    /**
     * Context's progress monitor.
     * Each context has it's progress monitor which is passed at context creation time and never changes.
     * @return progress monitor
     */
    @NotNull
    DBRProgressMonitor getProgressMonitor();

    /**
     * Context's purpose
     * @return purpose
     */
    @NotNull
    DBCExecutionPurpose getPurpose();

    /**
     * Prepares statements
     */
    @NotNull
    DBCStatement prepareStatement(
        @NotNull DBCStatementType type,
        @NotNull String query,
        boolean scrollable,
        boolean updatable,
        boolean returnGeneratedKeys) throws DBCException;

    boolean isLoggingEnabled();
    /**
     * Enables/disables operations logging within this session
     * @param enable enable
     */
    void enableLogging(boolean enable);
}
